//
//  MoviePlayerView.m
//

#import "MoviePlayerView.h"
#import "MoviePlayerDataProvider.h"
#import "FileURLHelper.h"
#ifdef IS_APPDOO_PREVIEW
#import "Prototype.h"
#import "ProjectResourcesCache.h"
#endif

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"

@implementation MoviePlayerView {
	NSTimer * _publishCurrentStateTimer;
	BOOL _isPlaying;
	BOOL _didPrepare;
}

- (BOOL)isPlaying {
	return _isPlaying;
}

- (void)awakeFromNib {
	[super awakeFromNib];
	[self prepare];
	[self publishCurrentState];
}
	
- (void)dealloc {
	if (_moviePlayer) [self unsetMoviePlayer];
}

- (void)removeFromSuperview {
	[super removeFromSuperview];
	if (_moviePlayer) [self unsetMoviePlayer];
}
	
- (void)setMovieURLString:(NSString *)movieURLString {
	_movieURLString = [movieURLString copy];
#ifndef IS_APPDOO_PREVIEW
	NSURL * url = GetFileURLFromString(self.movieURLString);
	if (!url) return;
#endif
#ifdef IS_APPDOO_PREVIEW
	// cache file:
	NSURL * url = nil;
	if ([_movieURLString hasPrefix:@"file:"]) {
		NSString * fileID = [[Prototype sharedInstance] resourceIDForFilename:[_movieURLString substringFromIndex:5]];
		if ([ProjectResourcesCache sharedCache] != nil && [[ProjectResourcesCache sharedCache] resourceWithIDIsCached:fileID]) {
			NSData * data = [[ProjectResourcesCache sharedCache] dataForResourceID:fileID];
			NSArray * paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
			NSString * cacheFile = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileID];
			NSError *cacheError = nil;
			[data writeToFile:cacheFile options:NSDataWritingAtomic error:&cacheError];
			if(!cacheError) {
				url = [NSURL fileURLWithPath:cacheFile];
			}
		}
	}
	if (!url) return;
#endif
	
	if (!self.moviePlayer) {
		_moviePlayer =  [[MPMoviePlayerController alloc] initWithContentURL:url];
		NSArray *backgroundModes = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"UIBackgroundModes"];
		if (backgroundModes && [backgroundModes isKindOfClass:[NSArray class]] && [backgroundModes containsObject:@"audio"]) {
			[_moviePlayer setUseApplicationAudioSession:NO];
		}
		_moviePlayer.view.frame = self.bounds;
		_moviePlayer.view.backgroundColor = self.backgroundColor;
		_moviePlayer.backgroundView.backgroundColor = self.backgroundColor;
		_moviePlayer.view.autoresizingMask = 18;
		self.showsControls = self.showsControls;
		self.autoplay = self.autoplay;
		self.repeat = self.repeat;
		[self prepare];
		[_moviePlayer prepareToPlay];
	}
	else {
		[_moviePlayer stop];
		_moviePlayer.contentURL = url;
		[_moviePlayer prepareToPlay];
	}

}

- (void)setShowsControls:(BOOL)showsControls {
	_showsControls = showsControls;
	_moviePlayer.controlStyle = showsControls ? MPMovieControlStyleDefault : MPMovieControlStyleNone;
}

- (void)setAutoplay:(BOOL)autoplay {
	_autoplay = autoplay;
	_moviePlayer.shouldAutoplay = autoplay;
}
	
- (void)setRepeat:(BOOL)repeat {
	_repeat = repeat;
	_moviePlayer.repeatMode = (repeat ? MPMovieRepeatModeOne : MPMovieRepeatModeNone);
}

- (void)unsetMoviePlayer {
	
    [[NSNotificationCenter defaultCenter] removeObserver:self
												 name:MPMoviePlayerPlaybackDidFinishNotification
											   object:_moviePlayer];

	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:MPMoviePlayerPlaybackStateDidChangeNotification
												  object:_moviePlayer];

	[self.moviePlayer stop];
	[self.moviePlayer.view removeFromSuperview];
	self.moviePlayer = nil;
	if (_publishCurrentStateTimer) [_publishCurrentStateTimer invalidate];
	_publishCurrentStateTimer = nil;

	[[NSNotificationCenter defaultCenter] postNotificationName:@"AudioPlayerDidDisappear" object:self];
#ifdef IS_APPDOO_PREVIEW
	// remove cached media file:
	if ([_movieURLString hasPrefix:@"file:"]) {
		NSString * fileID = [[Prototype sharedInstance] resourceIDForFilename:[_movieURLString substringFromIndex:5]];
		NSArray * paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
		NSString * cacheFile = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileID];
		if([[NSFileManager defaultManager] isReadableFileAtPath:cacheFile]) {
			[[NSFileManager defaultManager] removeItemAtPath:cacheFile error:nil];
		}
	}
#endif
	
}

- (void)prepare {
	if (!_moviePlayer || _didPrepare) return;
	_didPrepare = YES;
	
    [[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(moviePlaybackDidFinish:)
												 name:MPMoviePlayerPlaybackDidFinishNotification
											   object:_moviePlayer];
	
    [[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(moviePlaybackStateDidChange:)
												 name:MPMoviePlayerPlaybackStateDidChangeNotification
											   object:_moviePlayer];

    [self addSubview:_moviePlayer.view];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AudioPlayerDidPrepare" object:self];
}

- (void)publishCurrentState {
	[self.dataProvider updateContents];
}

- (void)moviePlaybackDidFinish:(NSNotification*)notification {
	
    MPMoviePlayerController *player = [notification object];
	if (player.isFullscreen) {
		[player setFullscreen:NO animated:YES];
	}
}

- (void)moviePlaybackStateDidChange:(NSNotification*)notification {
    MPMoviePlaybackState playbackState = [_moviePlayer playbackState];
	if (playbackState == MPMoviePlaybackStatePlaying) {
		[self beginPublishingCurrentState];
		_isPlaying = YES;
	}
	if (playbackState == MPMoviePlaybackStateInterrupted || playbackState == MPMoviePlaybackStatePaused || playbackState == MPMoviePlaybackStateStopped) {
		[self endPublishingCurrentState];
		_isPlaying = NO;
	}
	[self publishCurrentState];
}
		
- (void)beginPublishingCurrentState {
	if (_publishCurrentStateTimer) [self endPublishingCurrentState];
	_publishCurrentStateTimer = [NSTimer scheduledTimerWithTimeInterval:0.25 target:self selector:@selector(publishCurrentState) userInfo:nil repeats:YES];

}

- (void)endPublishingCurrentState {
	[self publishCurrentState];
	[_publishCurrentStateTimer invalidate];
	_publishCurrentStateTimer = nil;
}


- (void)play {
	[self.moviePlayer play];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AudioPlayerDidPrepare" object:self];
}

- (void)pause {
	[self.moviePlayer pause];
}

- (void)pauseOrPlay {
	if (_isPlaying) {
		[_moviePlayer pause];
	}
	else {
		[_moviePlayer play];
	}
}

- (void)stop {
	[self.moviePlayer stop];
}

- (void)toggleFullScreen {
	if (_moviePlayer.isFullscreen) {
		[_moviePlayer setFullscreen:NO animated:YES];
	}
	else {
		[_moviePlayer setFullscreen:YES animated:YES];
	}
}

- (void)seekBackward {
	_moviePlayer.currentPlaybackTime -= 5;
	
}

- (void)seekForward {
	_moviePlayer.currentPlaybackTime += 5;
}

- (void)beginSeekingBackward {
	[_moviePlayer beginSeekingBackward];
}

- (void)beginSeekingForward {
	[_moviePlayer beginSeekingForward];
}

- (void)endSeeking {
	[_moviePlayer endSeeking];
}

- (void)setDataProvider:(MoviePlayerDataProvider *)dataProvider {
	dataProvider.playerView = self;
	_dataProvider = dataProvider;
}

- (void)destroy {
	[self unsetMoviePlayer];
}

- (void)viewWillAppear:(BOOL)animated {
	if (!_moviePlayer.isPreparedToPlay) [_moviePlayer prepareToPlay];
}

@end
#pragma GCC diagnostic pop
